@@ -16,9 +16,10 @@
  *
  */
 
-#include "e-mapi-backend.h"
-
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
+
 #include <glib/gi18n-lib.h>
 
 #include <e-mapi-connection.h>
@@ -27,6 +28,9 @@
 #include <e-source-mapi-folder.h>
 #include <camel-mapi-settings.h>
 
+#include "e-mapi-backend-authenticator.h"
+#include "e-mapi-backend.h"
+
 #define E_MAPI_BACKEND_GET_PRIVATE(obj) \
 	(G_TYPE_INSTANCE_GET_PRIVATE \
 	((obj), E_TYPE_MAPI_BACKEND, EMapiBackendPrivate))
@@ -315,18 +319,23 @@ mapi_backend_source_changed_cb (ESource *source,
 static void
 mapi_backend_constructed (GObject *object)
 {
+	EBackend *backend = E_BACKEND (object);
 	ESource *source;
 
 	/* Chain up to parent's constructed() method. */
 	G_OBJECT_CLASS (e_mapi_backend_parent_class)->constructed (object);
 
-	source = e_backend_get_source (E_BACKEND (object));
+	source = e_backend_get_source (backend);
 
 	/* XXX Wondering if we ought to delay this until after folders
 	 *     are initially populated, just to remove the possibility
 	 *     of weird races with clients trying to create folders. */
 	e_server_side_source_set_remote_creatable (
 		E_SERVER_SIDE_SOURCE (source), TRUE);
+
+	g_signal_connect (
+		source, "changed",
+		G_CALLBACK (mapi_backend_source_changed_cb), backend);
 }
 
 static void
@@ -375,10 +384,6 @@ mapi_backend_populate (ECollectionBackend *backend)
 	 * hierarchy immediately on startup, schedule an authentication
 	 * session first thing. */
 	mapi_backend_queue_auth_session (mapi_backend);
-
-	g_signal_connect (
-		source, "changed",
-		G_CALLBACK (mapi_backend_source_changed_cb), backend);
 }
 
 static gchar *
@@ -499,6 +504,72 @@ mapi_backend_child_removed (ECollectionBackend *backend,
 		child_removed (backend, child_source);
 }
 
+static gboolean
+mapi_backend_create_resource_cb (EBackend *backend,
+				 CamelMapiSettings *settings,
+				 EMapiConnection *conn,
+				 gpointer user_data,
+				 GCancellable *cancellable,
+				 GError **error)
+{
+	ESourceBackend *backend_ext = NULL;
+	const gchar *folder_type_str = NULL;
+	ESource *source = user_data;
+	ESourceMapiFolder *folder_ext;
+	mapi_object_t obj_folder;
+	const gchar *foreign_username;
+	gboolean res = FALSE;
+	guint64 fid;
+
+	g_return_val_if_fail (e_source_has_extension (source, E_SOURCE_EXTENSION_MAPI_FOLDER), FALSE);
+
+	folder_ext = e_source_get_extension (source, E_SOURCE_EXTENSION_MAPI_FOLDER);
+	foreign_username = e_source_mapi_folder_get_foreign_username (folder_ext);
+
+	fid = e_source_mapi_folder_get_id (folder_ext);
+	g_return_val_if_fail (fid == 0, FALSE);
+
+	if (e_source_has_extension (source, E_SOURCE_EXTENSION_ADDRESS_BOOK)) {
+		backend_ext = e_source_get_extension (source, E_SOURCE_EXTENSION_ADDRESS_BOOK);
+		folder_type_str = IPF_CONTACT;
+	} else if (e_source_has_extension (source, E_SOURCE_EXTENSION_CALENDAR)) {
+		backend_ext = e_source_get_extension (source, E_SOURCE_EXTENSION_CALENDAR);
+		folder_type_str = IPF_APPOINTMENT;
+	} else if (e_source_has_extension (source, E_SOURCE_EXTENSION_TASK_LIST)) {
+		backend_ext = e_source_get_extension (source, E_SOURCE_EXTENSION_TASK_LIST);
+		folder_type_str = IPF_TASK;
+	} else if (e_source_has_extension (source, E_SOURCE_EXTENSION_MEMO_LIST)) {
+		backend_ext = e_source_get_extension (source, E_SOURCE_EXTENSION_MEMO_LIST);
+		folder_type_str = IPF_STICKYNOTE;
+	}
+
+	if (!backend_ext || g_strcmp0 (e_source_backend_get_backend_name (backend_ext), "mapi") != 0)
+		return FALSE;
+
+	fid = e_source_mapi_folder_get_parent_id (folder_ext);
+
+	if (foreign_username && *foreign_username)
+		res = e_mapi_connection_open_foreign_folder (conn, foreign_username, fid, &obj_folder, cancellable, error);
+	else if (e_source_mapi_folder_is_public (folder_ext))
+		res = e_mapi_connection_open_public_folder (conn, fid, &obj_folder, cancellable, error);
+	else
+		res = e_mapi_connection_open_personal_folder (conn, fid, &obj_folder, cancellable, error);
+
+	if (res) {
+		fid = 0;
+		if (!e_mapi_connection_create_folder (conn, &obj_folder, e_source_get_display_name (source), folder_type_str, &fid, cancellable, error))
+			fid = 0;
+		e_mapi_connection_close_folder (conn, &obj_folder, cancellable, error);
+
+		if (fid)
+			e_source_mapi_folder_set_id (folder_ext, fid);
+		else
+			res = FALSE;
+	}
+
+	return res;
+}
+
 static gboolean
 mapi_backend_create_resource_sync (ECollectionBackend *backend,
                                    ESource *source,
@@ -507,6 +578,9 @@ mapi_backend_create_resource_sync (ECollectionBackend *backend,
 {
 	ESourceRegistryServer *server;
 	ESource *parent_source;
+	CamelMapiSettings *settings;
+	ESourceMapiFolder *folder_ext;
+	const gchar *foreign_username;
 	const gchar *cache_dir;
 	const gchar *parent_uid;
 
@@ -519,8 +593,17 @@ mapi_backend_create_resource_sync (ECollectionBackend *backend,
 		return FALSE;
 	}
 
-	/* UI part takes care of the remote folder creation, if needed,
-	   thus just accept the source here */
+	settings = mapi_backend_get_settings (E_MAPI_BACKEND (backend));
+	g_return_val_if_fail (settings != NULL, FALSE);
+
+	folder_ext = e_source_get_extension (source, E_SOURCE_EXTENSION_MAPI_FOLDER);
+	foreign_username = e_source_mapi_folder_get_foreign_username (folder_ext);
+
+	if (!e_source_mapi_folder_is_public (folder_ext) &&
+	    !(foreign_username && *foreign_username) &&
+	    !e_mapi_backend_authenticator_run (
+		E_BACKEND (backend), settings, mapi_backend_create_resource_cb, source, cancellable, error))
+		return FALSE;
 
 	/* Configure the source as a collection member. */
 	parent_source = e_backend_get_source (E_BACKEND (backend));
@@ -545,6 +628,38 @@ mapi_backend_create_resource_sync (ECollectionBackend *backend,
 	return TRUE;
 }
 
+static gboolean
+mapi_backend_delete_resource_cb (EBackend *backend,
+				 CamelMapiSettings *settings,
+				 EMapiConnection *conn,
+				 gpointer user_data,
+				 GCancellable *cancellable,
+				 GError **error)
+{
+	ESource *source = user_data;
+	ESourceMapiFolder *folder_ext;
+	mapi_object_t *obj_store = NULL;
+	const gchar *foreign_username;
+	gboolean res = FALSE;
+	guint64 fid;
+
+	g_return_val_if_fail (e_source_has_extension (source, E_SOURCE_EXTENSION_MAPI_FOLDER), FALSE);
+
+	folder_ext = e_source_get_extension (source, E_SOURCE_EXTENSION_MAPI_FOLDER);
+	g_return_val_if_fail (!e_source_mapi_folder_is_public (folder_ext), FALSE);
+
+	foreign_username = e_source_mapi_folder_get_foreign_username (folder_ext);
+	g_return_val_if_fail (!foreign_username || !*foreign_username, FALSE);
+
+	fid = e_source_mapi_folder_get_id (folder_ext);
+	g_return_val_if_fail (fid != 0, FALSE);
+
+	if (e_mapi_connection_peek_store (conn, FALSE, NULL, &obj_store, cancellable, error))
+		res = e_mapi_connection_remove_folder (conn, obj_store, fid, cancellable, error);
+
+	return res;
+}
+
 static gboolean
 mapi_backend_delete_resource_sync (ECollectionBackend *backend,
                                    ESource *source,
@@ -552,6 +667,9 @@ mapi_backend_delete_resource_sync (ECollectionBackend *backend,
                                    GError **error)
 {
 	ESourceRegistryServer *server;
+	CamelMapiSettings *settings;
+	ESourceMapiFolder *folder_ext;
+	const gchar *foreign_username;
 
 	if (!e_source_has_extension (source, E_SOURCE_EXTENSION_MAPI_FOLDER)) {
 		g_set_error (
@@ -562,8 +680,18 @@ mapi_backend_delete_resource_sync (ECollectionBackend *backend,
 		return FALSE;
 	}
 
-	/* respective backends are taking care of cache removal
-	   same as remote folder removal, if needed */
+	settings = mapi_backend_get_settings (E_MAPI_BACKEND (backend));
+	g_return_val_if_fail (settings != NULL, FALSE);
+
+	folder_ext = e_source_get_extension (source, E_SOURCE_EXTENSION_MAPI_FOLDER);
+	foreign_username = e_source_mapi_folder_get_foreign_username (folder_ext);
+
+	if (!e_source_mapi_folder_is_public (folder_ext) &&
+	    !(foreign_username && *foreign_username) &&
+	    !e_mapi_backend_authenticator_run (
+		E_BACKEND (backend), settings, mapi_backend_delete_resource_cb, source, cancellable, error))
+		return FALSE;
+
 	server = e_collection_backend_ref_server (backend);
 	e_source_registry_server_remove_source (server, source);
 	g_object_unref (server);
